#include <PalmOS.h>
#include "VFSMgr.h"

#define MAX_PATH				4096
#define MEM_COPY_CHUNK			1024
#define BAD_ATTRS				(vfsFileAttrVolumeLabel | vfsFileAttrDirectory | vfsFileAttrLink)

#define SYS_TONE_DB_NAME		"SndFile Ring Tones"
#define SYS_TONE_DB_CRID		'HsTo'
#define SYS_TONE_DB_TYPE		'SndI'
#define SYS_MP3_TONE_DB_TYPE	'MP3f'


static FileRef prvOpenFileOnAnyCard(const char* path){
	
	UInt16 vrn;
	UInt32 vi = vfsIteratorStart;
	FileRef f;
	UInt32 attr;
	
	while(vi != vfsIteratorStop){ 
		
		if(errNone == VFSVolumeEnumerate(&vrn, &vi)){ 
			
			if(errNone == VFSFileOpen(vrn,path,vfsModeRead,&f)){
				
				VFSFileGetAttributes(f,&attr);
				
				if(attr & BAD_ATTRS){
					
					VFSFileClose(f);
					f = NULL;
				}
				
				return f;
			}
		}
		else{ 
			
			break;
		} 
	}
	
	return NULL;
}

static inline Boolean prvAddToSystemToneDb(const char* name){
	
	UInt16 sz = StrLen(name) + 1;
	LocalID LID;
	DmOpenRef db;
	Boolean ret = false;
	
	
	//create the DB if not present yet and open it
	
	DmCreateDatabase(0,SYS_TONE_DB_NAME,SYS_TONE_DB_CRID,SYS_TONE_DB_TYPE,false);
	
	LID = DmFindDatabase(0,SYS_TONE_DB_NAME);
	
	if(LID){
		
		db = DmOpenDatabase(0,LID,dmModeReadWrite);
		
		if(db){
			
			MemHandle rec;
			UInt16 at = 0xFFFF;
			
			rec = DmNewRecord(db,&at,sz);
			
			if(rec){
				
				DmWrite(MemHandleLock(rec),0,name,sz);
				MemHandleUnlock(rec);
				DmReleaseRecord(db,at,true);
				
				ret = true;
			}
			
			DmCloseDatabase(db);
		}
	}
	
	return ret;
}

static inline Boolean prvCopyData(FileHand to,FileRef from){
	
	UInt8* buf;
	Boolean ret = false;
	UInt32 sz,origSz,done,now;
	char progress[32];
	
	buf = MemPtrNew(MEM_COPY_CHUNK);
	
	if(buf){
		
		VFSFileSize(from,&sz);
		origSz = sz;
		
		while(sz){
			
			now = sz;
			if(now > MEM_COPY_CHUNK) now = MEM_COPY_CHUNK;
			
			VFSFileRead(from,now,buf,&now);
			if(!now) break;
			
			done = FileWrite(to,buf,1,now,NULL);
			if(!done) break;
			
			if(done != now){
				
				VFSFileSeek(from,vfsOriginCurrent,done - now);
			}
			
			sz -= done;
			
			WinDrawChars(progress,StrPrintF(progress,"%ld/%ld done          ",origSz - sz, origSz),0,149);
		}
		
		ret = (sz == 0);
		
		MemPtrFree(buf);
	}
	
	return ret;
}

static Boolean prvImportRingtone(const char* path){
	
	FileRef f;
	FileHand s;
	Boolean ret = false;
	
	f = prvOpenFileOnAnyCard(path);
	
	if(f){
		
		char dbName[32];
		const char* fileName = path + StrLen(path);
		
		
		//get filename from path
		
		while(fileName != path && *fileName != '\\' && *fileName != '/') fileName--;
		fileName++;
		
		
		//make a palm DB name
		
		StrCopy(dbName,"Ton-");
		StrNCat(dbName,fileName,sizeof(dbName));
		
		
		//try to make a FileStream with that name
		
		s = FileOpen(0,dbName,SYS_MP3_TONE_DB_TYPE,SYS_TONE_DB_CRID,fileModeReadWrite,NULL);
		
		if(s){
			
			if(prvCopyData(s,f)){
				
				ret = prvAddToSystemToneDb(dbName);
			}
			
			FileClose(s);
			
			if(!ret) FileDelete(0,dbName);
		}
		
		VFSFileClose(f);
	}
	return ret;
}

static Boolean prvCheckResponce(Int16 canceled,char *attempt){
	
	FileRef f;
	
	if(!canceled){
		
		f = prvOpenFileOnAnyCard(attempt);
		if(f){
			
			VFSFileClose(f);
			return true;
		}
		else{
			
			FrmAlert(1002);
			return false;
		}
	}
	
	return true;
}

UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	if(cmd == 0){
		
		char* path;
		UInt16 canceled;
		
		path = MemPtrNew(MAX_PATH);
		
		if(path){
			
			
			while(1){
				
				canceled = FrmCustomResponseAlert(1000,NULL,NULL,NULL,path,MAX_PATH - 1,&prvCheckResponce);
				
				
				if(canceled) break;
				
				FrmCustomAlert(1001,prvImportRingtone(path)?"Succes":"Failure",NULL,NULL);
			}
			
			MemPtrFree(path);
		}
	}
	
	return 0;
}
